package conm.zhentao.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import conm.zhentao.dto.OrderCreateDTO;
import conm.zhentao.dto.OrderItemDTO;
import conm.zhentao.entity.OrderDetail;
import conm.zhentao.entity.Order_info;
import conm.zhentao.entity.Sku;
import conm.zhentao.exception.BusinessException;
import conm.zhentao.mapper.OrderDetailMapper;
import conm.zhentao.mapper.OrderMapper;
import conm.zhentao.service.CouponService;
import conm.zhentao.service.OrderService;
import conm.zhentao.service.SkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order_info> implements OrderService {

    @Autowired
    private SkuService skuService;

    @Autowired
    private CouponService couponService;

    @Autowired
    private OrderDetailMapper orderDetailMapper;

    @Override
    @Transactional
    public Order_info createOrder(OrderCreateDTO orderCreateDTO, Long userId) {
        // 1. 获取商品信息
        List<Long> skuIds = orderCreateDTO.getItems().stream()
                .map(OrderItemDTO::getSkuId)
                .collect(Collectors.toList());
        Map<Long, Sku> skuMap = skuService.listByIds(skuIds).stream()
                .collect(Collectors.toMap(Sku::getId, sku -> sku));

        // 2. 创建订单
        Order_info order = new Order_info();
        order.setUserId(userId);
        order.setOrderNo(generateOrderNo());
        order.setStatus(1);
        order.setCreateTime(LocalDateTime.now());
        order.setUpdateTime(LocalDateTime.now());

        // 3. 创建订单明细并计算总金额
        List<OrderDetail> orderDetails = new ArrayList<>();
        BigDecimal totalAmount = BigDecimal.ZERO;

        for (OrderItemDTO item : orderCreateDTO.getItems()) {
            Sku sku = skuMap.get(item.getSkuId());
            if (sku == null) {
                throw new BusinessException(400, "商品不存在: " + item.getSkuId());
            }

            if (sku.getStock() < item.getQuantity()) {
                throw new BusinessException(400, "商品库存不足: " + sku.getName());
            }

            OrderDetail detail = new OrderDetail();
            detail.setSkuId(sku.getId());
            detail.setSkuName(sku.getName());
            detail.setPrice(sku.getPrice());
            detail.setQuantity(item.getQuantity());
            detail.setTotalAmount(sku.getPrice().multiply(new BigDecimal(item.getQuantity())));
            detail.setCreateTime(LocalDateTime.now());
            detail.setUpdateTime(LocalDateTime.now());
            
            orderDetails.add(detail);
            totalAmount = totalAmount.add(detail.getTotalAmount());
        }

        // 4. 计算优惠金额
        BigDecimal discountAmount = BigDecimal.ZERO;
        if (orderCreateDTO.getCouponId() != null) {
            try {
                discountAmount = couponService.calculateDiscount(orderCreateDTO.getCouponId(), totalAmount);
                order.setCouponId(orderCreateDTO.getCouponId());
            } catch (Exception e) {
                throw new BusinessException(400, "优惠券使用失败: " + e.getMessage());
            }
        }

        // 5. 设置订单金额
        order.setTotalAmount(totalAmount);
        order.setDiscountAmount(discountAmount);
        order.setActualAmount(totalAmount.subtract(discountAmount));

        // 6. 保存订单和订单明细
        save(order);
        for (OrderDetail detail : orderDetails) {
            detail.setOrderId(order.getId());
            orderDetailMapper.insert(detail);
        }

        // 7. 使用优惠券
        if (orderCreateDTO.getCouponId() != null) {
            try {
                couponService.useCoupon(orderCreateDTO.getCouponId(), userId);
            } catch (Exception e) {
                throw new BusinessException(400, "优惠券使用失败: " + e.getMessage());
            }
        }

        return order;
    }

    private String generateOrderNo() {
        // 生成订单号的逻辑，可以使用时间戳+随机数
        return "ORDER" + System.currentTimeMillis() + (int)(Math.random() * 1000);
    }
} 